{Copyright (C) [2003] [Benito van der Zander]
All rights reserved.

How you can contact me:
  Mail: BeniBela@aol.com
  WWW:  http://www.benibela.de
  
This program is distributed under the terms of the MPL license 1.1:

"The contents of this file are subject to the Mozilla Public License
Version 1.1 (the "License"); you may not use this file except in
compliance with the License. You may obtain a copy of the License at
http://www.mozilla.org/MPL/

Software distributed under the License is distributed on an "AS IS"
basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
License for the specific language governing rights and limitations
under the License.

The Original Code is huffman.pas.

The Initial Developer of the Original Code is Benito van der Zander.
All Rights Reserved.

Alternatively, the contents of this file may be used under the terms
of the Gnu Public License Version 2 or higher license (the  "GPL 2 License"), in which case the
provisions of GPL 2 License are applicable instead of those
above. If you wish to allow use of your version of this file only
under the terms of the GPL 2 License and not to allow others to use
your version of this file under the MPL, indicate your decision by
deleting the provisions above and replace them with the notice and
other provisions required by the GPL 2 License. If you do not delete
the provisions above, a recipient may use your version of this file
under either the MPL or the GPL 2 License."}

unit huffman;

interface
{$DEFINE NonVCL} //VCL ist sehr langsamer
//{$DEFINE optimizer} //Mit Optimizer wird die bentigte Zeit ausgeben
{$IFDEF optimizer}
  uses dialogs,windows,sysutils;
{$ENDIF}
type
  TDynByteArray=array of byte;
function HuffmanEncodeString(s:AnsiString):TDynByteArray;
function HuffmanDecodeDynByteArray(dynAra:TDynByteArray):AnsiString;
function HuffmanEncodeStringW(s:WideString):TDynByteArray;
function HuffmanDecodeDynByteArrayW(dynAra:TDynByteArray):WideString;

function HuffmanReadCompressedAnsiStringFromFile(datei: string):AnsiString;
procedure HuffmanWriteAnsiStringToFile(str:AnsiString;datei:string);

procedure HuffmanCompressFile(datei:string);
procedure HuffmanDeCompressFile(datei:string);
implementation
{$IFNDEF NonVcl}
uses
  classes;
{$ENDIF}
type PBinaerTreeElement=^TBinaerTreeElement;
     TBinaerTreeElement=record
       Zweig0,Zweig1:PBinaerTreeElement;
       Zeichen:AnsiChar;
       haeufigkeit:integer;
       blatt:boolean;
     end;
     TBitCode=array[1..256] of boolean;
     TBitCodeForChar=record
       bitCode:TBitCode;
       bitCount:byte;
     end;
     TArrayOfBitCodeForChar=array[0..255] of TBitCodeForChar;
{$IFDEF NonVcl}
     {}{}{}{}{}{}{}{}{}{}{}{}{}{}{}{}{}{}{}{}{}{}{}{}{}{}{}{}{}{}{}{}{}{}{}{}{}{
     ###########################################################################
     ===========================================================================
     ---------------------------------------------------------------------------
     ..............................Begin TBits..................................
     ---------------------------------------------------------------------------
     ===========================================================================
     ###########################################################################
     {}{}{}{}{}{}{}{}{}{}{}{}{}{}{}{}{}{}{}{}{}{}{}{}{}{}{}{}{}{}{}{}{}{}{}{}{{}
     TBits = class    {kopiert aus classes.pas, um nicht die VCL benutzten zu mssen}
     private
       FSize: Integer;
       FBits: Pointer;
       procedure Error;
       procedure SetSize(const Value: Integer);
       procedure SetBit(Index: Integer; const Value: Boolean);
       function GetBit(Index: Integer): Boolean;
     public
       destructor Destroy; override;
       function OpenBit: Integer;
       property Bits[Index: Integer]: Boolean read GetBit write SetBit; default;
       property Size: Integer read FSize write SetSize;
     end;
  const
    BitsPerInt = SizeOf(Integer) * 8;

  type
    TBitEnum = 0..BitsPerInt - 1;
    TBitSet = set of TBitEnum;
    PBitArray = ^TBitArray;
    TBitArray = array[0..4096] of TBitSet;

  destructor TBits.Destroy;
  begin
    SetSize(0);
    inherited Destroy;
  end;

  procedure TBits.Error;
  begin
  end;

  procedure TBits.SetSize(const Value: Integer);
  var
    NewMem: Pointer;
    NewMemSize: Integer;
    OldMemSize: Integer;
  
    function Min(const X, Y: Integer): Integer;
    begin
      Result := X;
      if X > Y then Result := Y;
    end;

  begin
    if Value <> Size then
    begin
      if Value < 0 then Error;
      NewMemSize := ((Value + BitsPerInt - 1) div BitsPerInt) * SizeOf(Integer);
      OldMemSize := ((Size + BitsPerInt - 1) div BitsPerInt) * SizeOf(Integer);
      if NewMemSize <> OldMemSize then
      begin
        NewMem := nil;
        if NewMemSize <> 0 then
        begin
          GetMem(NewMem, NewMemSize);
          FillChar(NewMem^, NewMemSize, 0);
        end;
        if OldMemSize <> 0 then
        begin
          if NewMem <> nil then
            Move(FBits^, NewMem^, Min(OldMemSize, NewMemSize));
          FreeMem(FBits, OldMemSize);
        end;
        FBits := NewMem;
      end;
      FSize := Value;
    end;
  end;


  procedure TBits.SetBit(Index: Integer; const Value: Boolean); assembler;
  asm
          CMP     Index,[EAX].FSize
          JAE     @@Size
  
  @@1:    MOV     EAX,[EAX].FBits
          OR      Value,Value
          JZ      @@2
          BTS     [EAX],Index
          RET

  @@2:    BTR     [EAX],Index
          RET
  
  @@Size: CMP     Index,0
          JL      TBits.Error
          PUSH    Self
          PUSH    Index
          PUSH    ECX {Value}
          INC     Index
          CALL    TBits.SetSize
          POP     ECX {Value}
          POP     Index
          POP     Self
          JMP     @@1
  end;

  function TBits.GetBit(Index: Integer): Boolean; assembler;
  asm
          CMP     Index,[EAX].FSize
          JAE     TBits.Error
          MOV     EAX,[EAX].FBits
          BT      [EAX],Index
          SBB     EAX,EAX
          AND     EAX,1
  end;


  function TBits.OpenBit: Integer;
  var
    I: Integer;
    B: TBitSet;
    J: TBitEnum;
    E: Integer;
  begin
    E := (Size + BitsPerInt - 1) div BitsPerInt - 1;
    for I := 0 to E do
      if PBitArray(FBits)^[I] <> [0..BitsPerInt - 1] then
      begin
        B := PBitArray(FBits)^[I];
        for J := Low(J) to High(J) do
        begin
          if not (J in B) then
          begin
            Result := I * BitsPerInt + J;
            if Result >= Size then Result := Size;
              Exit;
          end;
        end;
      end;
    Result := Size;
  end;
     {}{}{}{}{}{}{}{}{}{}{}{}{}{}{}{}{}{}{}{}{}{}{}{}{}{}{}{}{}{}{}{}{}{}{}{}{}{
     ###########################################################################
     ===========================================================================
     ---------------------------------------------------------------------------
     ..............................Ende TBits..................................
     ---------------------------------------------------------------------------
     ===========================================================================
     ###########################################################################
     {}{}{}{}{}{}{}{}{}{}{}{}{}{}{}{}{}{}{}{}{}{}{}{}{}{}{}{}{}{}{}{}{}{}{}{}{{}
{$ENDIF}

//Es wird davon ausgegangen, das bitCode leer ist (alles false)
procedure Tiefensuche(var liste:TArrayOfBitCodeForChar; const baum:TBinaerTreeElement;const bitCode:TBitCode;const bitCount:byte);
var temp:TBitCode;
begin
  if baum.blatt then begin
    if bitCount=0 then begin
      liste[ord(baum.Zeichen)].bitCode[1]:=false;
      liste[ord(baum.Zeichen)].bitCount:=1;
    end else begin
      liste[ord(baum.Zeichen)].bitCode:=bitCode;
      liste[ord(baum.Zeichen)].bitCount:=bitCount;
    end;
  end else begin

    if baum.Zweig0<>nil then Tiefensuche(liste,baum.Zweig0^,bitCode,bitCount+1);
    if baum.Zweig1<>nil then begin
      Move(bitCode, temp,sizeof( temp));
      temp[bitCount+1]:=true;
      Tiefensuche(liste,baum.Zweig1^,temp,bitCount+1);
    end;
  end;
end;
  procedure DeleteBaum(baum:PBinaerTreeElement);
  begin
    if baum^.Zweig0<>nil then DeleteBaum(baum^.zweig0);
    if baum^.Zweig1<>nil then DeleteBaum(baum^.zweig1);
    dispose(baum);
  end;
{$IFDEF optimizer}
  var time:cardinal;
procedure GetTime;
begin
  time:=GetTickCount;
end;
procedure PaintTime(s:string);
begin
  ShowMessage(s+INtToStr(getTickCount-time));
end;
{$ENDIF}
function HuffmanEncodeString(s:string):TDynByteArray;
var
    List:array of TBinaerTreeElement;
    ListPos:integer;
  procedure AddToList(Zeichen:char;haeufigkeit:integer);
  begin
    if ListPos=length(list) then setLength(list,length(list)*2);
    list[listpos].Zeichen:=Zeichen;
    List[ListPos].haeufigkeit:=haeufigkeit;
    List[ListPos].Zweig0:=nil;
    List[ListPos].Zweig1:=nil;
    List[ListPos].blatt:=true;
    inc(listPos);
  end;
  procedure AddToListWithZweig(Zeichen:char;haeufigkeit:integer;zweig0:TBinaerTreeElement;zweig1:TBinaerTreeElement);
  begin
    if ListPos=length(list) then setLength(list,length(list)*2);
    list[listpos].Zeichen:=Zeichen;
    List[ListPos].haeufigkeit:=haeufigkeit;
    new(List[ListPos].Zweig0);
    List[ListPos].Zweig0^:=zweig0;
    new(List[ListPos].Zweig1);
    List[ListPos].Zweig1^:=zweig1;
    List[ListPos].blatt:=False;
    inc(listPos);
  end;
  procedure Delete(index:integer);
  begin
    dec(listPos);
    List[index]:=list[listpos];
  end;
var
  bits:TBits;
  usedSize:integer;
  procedure AddBits(bitCode:TBitCode;bitCount:byte);
  var i,oldsize:integer;
  begin
    oldsize:=bits.Size;
    if usedSize+bitCount>oldsize then begin
    if bitCount>oldsize then
      bits.Size:=oldSize+bitCount
     else
      bits.Size:=bits.Size*2;
    end;
    for i:=1 to bitCount do begin
      bits.Bits[usedSize]:=bitCode[i];
      inc(usedSize);
    end;
  end;
  procedure AddIntBits(bitCode:cardinal;bitCount:byte);
  var i,oldsize:integer;
  begin
    oldsize:=bits.Size;
    if usedSize+bitCount>oldsize then begin
    if bitCount>oldsize then
      bits.Size:=oldSize+bitCount
     else
      bits.Size:=bits.Size*2;
    end;
    for i:=0 to bitCount-1 do begin
      bits.Bits[usedSize]:=(bitcode and ($1 shl i))<>0;
      inc(usedSize);
    end;
  end;
  procedure SaveBaum(baum:TBinaerTreeElement);
  begin
    if baum.blatt then begin
      AddIntBits(1,1);
      AddIntBits(ord(baum.zeichen),8);
    end else begin
      AddIntBits(0,1);
      SaveBaum(baum.zweig0^);
      if baum.zweig1<>nil then begin
        AddIntBits(1,1);
        SaveBaum(baum.zweig1^);
      end else
        AddIntBits(0,1);
    end;
  end;
var
    Baum:TBinaerTreeElement;
var haeufigkeiten:array[0..255] of integer;
    //List:TListOfTreeElements;
    i:integer;
    Differenz:byte;
    Kleinstes,ZweitKleinstes:integer;
    BitCodeForChar:TArrayOfBitCodeForChar;
    tempbitcode:TBitCode;
begin
  //Initialisierung
  FillChar(haeufigkeiten,sizeof(haeufigkeiten),#00);
  SetLength(list,1);
  ListPos:=0;
  bits:=TBits.create;
  bits.Size:=0;
  usedSize:=0;
  FillChar(BitCodeForChar,sizeof(BitCodeForChar),#00);

  try
    {$IFDEF optimizer}GetTime;{$ENDIF}
    //Hufigkeiten berechnen
    for i:=1 to length(s) do
      inc(haeufigkeiten[ord(s[i])]);
    {$IFDEF optimizer}PaintTime('Berechnung der Hufigkeiten: ');{$ENDIF}

    //Alle Hufigkeiten in die Liste kopieren
    {$IFDEF optimizer}GetTime;{$ENDIF}
    for i:=0 to 255 do begin
      if haeufigkeiten[i]<>0 then begin
        AddToList(chr(i),haeufigkeiten[i]);
      end;
    end;
    {$IFDEF optimizer}PaintTime('Kopierung in die Liste: ');{$ENDIF}
    if ListPos=1 then
      AddToList(chr(integer(ord(list[0].zeichen)+1) mod 256),0);
      //Berechnet den Baum
    {$IFDEF optimizer}GetTime;{$ENDIF}
    while (ListPos>1) do begin
      Kleinstes:=0;
      for i:=1 to ListPos-1 do
        if List[i].haeufigkeit<=List[Kleinstes].haeufigkeit then
          Kleinstes:=i;
      if Kleinstes=1 then
        ZweitKleinstes:=0
       else
        ZweitKleinstes:=1;
      for i:=0 to ListPos-1 do
        if (List[i].haeufigkeit<=List[ZweitKleinstes].haeufigkeit)and(i<>Kleinstes) then
          ZweitKleinstes:=i;
      AddToListWithZweig(#0,list[Kleinstes].haeufigkeit+list[ZweitKleinstes].haeufigkeit,list[Kleinstes],list[ZweitKleinstes]);

      Delete(Kleinstes);
      if ZweitKleinstes=ListPos+1 then
        Delete(Kleinstes)
       else
        Delete(ZweitKleinstes);
    end;
    {$IFDEF optimizer}PaintTime ('Baumberechnung: ');{$ENDIF}
    Baum:=list[0];

    //AddIntBits(0,32); {32 Bits reservieren}
    bits.Size:=128;
    //64 Bits reservieren
    usedSize:=4; //erste 4 nehmen die Verschiebung an (es knnen ja nur ganze Bytes gespeichert werden)
    AddIntBits(length(s),32); //Gre des Textes
    usedSize:=usedSize+32; //Reserviert fr Erweiterungen

    {$IFDEF optimizer}GetTime;{$ENDIF}
    SaveBaum(baum);
    {$IFDEF optimizer}PaintTime('Baum speichern: ');;{$ENDIF}

    FillChar(tempbitcode,sizeof(tempbitcode),#00);
    Tiefensuche(BitCodeForChar,baum,tempbitcode,0);


    {$IFDEF optimizer}GetTime;{$ENDIF}
    for i:=1 to length(s) do begin
      AddBits(BitCodeForChar[ord(s[i])].bitCode,BitCodeForChar[ord(s[i])].bitCount);
    end;
    {$IFDEF optimizer}PaintTime('Bits hinzufgen: ');;{$ENDIF}
    Differenz:=usedSize mod 8;
    if Differenz <> 0 then Differenz:=8-Differenz;
    for i:=0 to 3 do begin
      bits[3-i]:=Differenz and ($1 shl i)<>0;
    end;
    usedsize:=usedsize+Differenz;
    if bits.size<usedsize then bits.size:=usedsize;
    SetLength(result,usedSize div 8);
    {$IFDEF NonVcl}
    Move(bits.FBits^,result[0],length(result));
    {$ELSE}
    bitPos:=0;
    for i:=0 to high(result) do begin
      Result[i]:=byte(bits[bitPos]) or (byte(bits[bitPos+1]) shl 1)or (byte(bits[bitPos+2]) shl 2)or (byte(bits[bitPos+3]) shl 3)or (byte(bits[bitPos+4]) shl 4)or (byte(bits[bitPos+5]) shl 5)or (byte(bits[bitPos+6]) shl 6)or (byte(bits[bitPos+7]) shl 7);
      inc(bitpos,8);
    end;
    {$ENDIF}
  finally
    //Finitialosion
    if baum.Zweig0<>nil then DeleteBaum(baum.zweig0);
    if baum.Zweig1<>nil then DeleteBaum(baum.zweig1);
    bits.Free;
  end;
end;


function HuffmanDecodeDynByteArray(dynAra:TDynByteArray):string;
var
    bits:TBits;
    bitPos:integer;
  procedure LoadBaum(var baum:TBinaerTreeElement);
  var temp_b:byte;
      temp_i:integer;
  begin
     case bits.Bits[bitPos] of
       true: begin
          inc(bitpos);
          temp_i:=0;
          for temp_b:=0 to 7 do begin
            if bits.Bits[bitpos] then temp_i:=temp_i or (1 shl temp_b);
            inc(bitpos);
          end;
          baum.Zweig0:=nil;
          baum.Zweig1:=nil;
          baum.Zeichen:=chr(temp_i mod 256);
          baum.blatt:=true;
       end;
       false: begin
         baum.blatt:=false;
         inc(bitpos);
         new(baum.zweig0);
         LoadBaum(baum.zweig0^);
         if bits.bits[bitPos] then begin
           inc(bitpos);
           new(baum.zweig1);
           LoadBaum(baum.zweig1^);
         end else begin
           inc(bitpos);
           baum.zweig1:=nil;
         end;
       end;
     end;
  end;
type TBitCodeAndChar=record
  bitCode:TBitCode;
  bitCount:byte;
  char:char;
end;
type TBitCodeForCharsForBitCount=array of TBitCodeAndChar;

var baum:TBinaerTreeElement;
    aktbaum:PBinaerTreeElement;
    BitCodeForChar:TArrayOfBitCodeForChar;
    BitCodeAndCharsForBitCount:array[0..31] of TBitCodeForCharsForBitCount;
    i,j:integer;
    aktwert:boolean;
    aktpos:integer;
    tempBits:TBitCode;
begin
  FillChar(BitCodeForChar,sizeof(BitCodeForChar),#00);
  bits:=TBits.create;
  bits.Size:=length(dynAra)*8;
  {$IFDEF NonVcl}
  Move(dynAra[0],bits.FBits^,length(dynAra));
  {$ELSE}
  bitPos:=0;
  for i:=0 to high(dynAra) do begin
    bits.Bits[bitPos]:=(dynAra[i]and $1) =$1;
    inc(bitpos);
    bits.Bits[bitPos]:=(dynAra[i]and $2) =$2;
    inc(bitpos);
    bits.Bits[bitPos]:=(dynAra[i]and $4) =$4;
    inc(bitpos);
    bits.Bits[bitPos]:=(dynAra[i]and $8) =$8;
    inc(bitpos);
    bits.Bits[bitPos]:=(dynAra[i]and $10) =$10;
    inc(bitpos);
    bits.Bits[bitPos]:=(dynAra[i]and $20) =$20;
    inc(bitpos);
    bits.Bits[bitPos]:=(dynAra[i]and $40) =$40;
    inc(bitpos);
    bits.Bits[bitPos]:=(dynAra[i]and $80) =$80;
    inc(bitpos);
  end;
  {$ENDIF}
  bitPos:=68; //Reservierte Bytes berspringen
  Result:='';
  for i:=0 to 31 do
    setLength(BitCodeAndCharsForBitCount[i],0);
  try
    LoadBaum(baum);
    FillChar(tempBits,sizeof(tempBits),#00);
    Tiefensuche(BitCodeForChar,baum,tempBits,0);

    for i:=0 to {255}127 do begin
      j:=i*2;
      if BitCodeForChar[j].bitCount<>0 then begin
        SetLength(BitCodeAndCharsForBitCount[BitCodeForChar[j].bitCount],length(BitCodeAndCharsForBitCount[BitCodeForChar[j].bitCount])+1);
        BitCodeAndCharsForBitCount[BitCodeForChar[j].bitCount][high(BitCodeAndCharsForBitCount[BitCodeForChar[j].bitCount])].bitCode:=BitCodeForChar[j].bitCode;
        BitCodeAndCharsForBitCount[BitCodeForChar[j].bitCount][high(BitCodeAndCharsForBitCount[BitCodeForChar[j].bitCount])].bitCount:=BitCodeForChar[j].bitCount;
        BitCodeAndCharsForBitCount[BitCodeForChar[j].bitCount][high(BitCodeAndCharsForBitCount[BitCodeForChar[j].bitCount])].char:=chr(j);
      end;
      inc(j);
      if BitCodeForChar[j].bitCount<>0 then begin
        SetLength(BitCodeAndCharsForBitCount[BitCodeForChar[j].bitCount],length(BitCodeAndCharsForBitCount[BitCodeForChar[j].bitCount])+1);
        BitCodeAndCharsForBitCount[BitCodeForChar[j].bitCount][high(BitCodeAndCharsForBitCount[BitCodeForChar[j].bitCount])].bitCode:=BitCodeForChar[j].bitCode;
        BitCodeAndCharsForBitCount[BitCodeForChar[j].bitCount][high(BitCodeAndCharsForBitCount[BitCodeForChar[j].bitCount])].bitCount:=BitCodeForChar[j].bitCount;
        BitCodeAndCharsForBitCount[BitCodeForChar[j].bitCount][high(BitCodeAndCharsForBitCount[BitCodeForChar[j].bitCount])].char:=chr(j);
      end;
    end;
    fillChar(aktwert,sizeof(aktwert),#00);
    aktbaum:=@baum;

    aktpos:=0;
    for i:=0 to 31 do
      aktpos:=aktpos or (byte(bits[36-i]) shl (31-i));
    setLength(result,aktpos);

    aktpos:=1;
    for i:=bitPos to bits.Size-1-((byte(bits[0]) shl 3) or (byte(bits[1]) shl 2) or (byte(bits[2]) shl 1) or byte(bits[3])) do begin
      aktwert:=bits.Bits[i];
      if aktwert then aktbaum:=aktbaum^.zweig1
      else aktbaum:=aktbaum^.zweig0;
      if aktbaum^.blatt then begin
        if aktpos>length(result) then setLength(result,length(result)*2);
        Result[aktpos]:=aktbaum^.Zeichen;
        inc(aktpos);
        aktbaum:=@baum;
      end;
    end;
    SetLength(result,aktpos-1);
  finally
    if baum.Zweig0<>nil then DeleteBaum(baum.zweig0);
    if baum.Zweig1<>nil then DeleteBaum(baum.zweig1);

    bits.free;
  end;
end;

function HuffmanEncodeStringW(s:WideString):TDynByteArray;
var ansi:AnsiString;
    temp_w:word;
    i,StringPos:integer;
begin
  SetLength(ansi,length(s)*2);
  StringPos:=1;
  for i:=1 to length(s) do begin
    temp_w:=ord(s[i]);
    ansi[StringPos]:=chr(temp_w div $FF);
    inc(StringPos);
    ansi[StringPos]:=chr(temp_w mod $FF);
    inc(StringPos);
  end;
  Result:=HuffmanEncodeString(ansi);
end;
function HuffmanDecodeDynByteArrayW(dynAra:TDynByteArray):WideString;
var ansi:AnsiString;
    temp_w:word;
    i:integer;
begin
  ansi:=HuffmanDecodeDynByteArray(dynAra);
  SetLength(result,length(ansi)div 2);
  for i:=0 to length(ansi)div 2-1 do begin
    temp_w:=ord(ansi[i*2]) * $FF + ord(ansi[i*2+1]);
    result[i]:=WideChar(temp_w);
  end;
end;

function HuffmanReadCompressedAnsiStringFromFile(datei: string):AnsiString;
var BlockFile:file;
    tempArray:TDynByteArray;
begin
AssignFile(BlockFile,datei);
Reset(BlockFile,1);
try
  SetLength(tempArray,FileSize(BlockFile));
  BlockRead(BlockFile,tempArray[0],length(tempArray));
  Result:=HuffmanDecodeDynByteArray(tempArray);
finally
  CloseFile(BlockFile);
end;
end;

procedure HuffmanWriteAnsiStringToFile(str:AnsiString;datei:string);
var BlockFile:file;
    tempArray:TDynByteArray;
begin
AssignFile(BlockFile,datei);
Rewrite(BlockFile,1);
try
  tempArray:=HuffmanEncodeString(str);
  BlockWrite(BlockFile,tempArray[0],length(tempArray));
finally
  CloseFile(BlockFile);
end;
end;

procedure HuffmanCompressFile(datei:string);
var text:File;
    tempStr:AnsiString;
begin
  //Datei lesen
  AssignFile(text,datei);
  Reset(text,1);
  try
    SetLength(tempStr,FileSize(text));
    BlockRead(text,tempStr[1],length(tempStr));
  finally
    CloseFile(text);
  end;

  //Datei schreiben
  HuffmanWriteAnsiStringToFile(tempStr,datei);
end;
procedure HuffmanDeCompressFile(datei:string);
var text:TextFile;
    tempStr:string;
begin
  //Datei lesen
  tempStr:=HuffmanReadCompressedAnsiStringFromFile(datei);

  //Datei schreiben
  AssignFile(text,datei);
  Rewrite(text);
  try
    write(text,tempStr);
  finally
    CloseFile(text);
  end;
end;
{     PListOfTreeElements=^TListOfTreeElements;
     TListOfTreeElements=record
       next:PListOfTreeElements;
       treeElement:PBinaerTreeElement;
     end;

procedure AddToListOfTreeElements(var list:TListOfTreeElements;newItem:TBinaerTreeElement);
begin
  new(list.next);
  list.next.treeElement:=newItem;
end;
procedure DeleteListItem(var prev,item:TListOfTreeElements);
begin
  prev.next:=item.next;
  Dispose(@item);
end;
procedure FreeList(var list:TListOfTreeElements);
var next,old:PListOfTreeElements;
begin
  next:=list.next;
  while (next<>nil) do begin
    old:=next;
    next:=next^.next;
    Dispose(old);
  end;
end;
                                              }

end.


